;   This program is based on the cascade loop configuration
;   available in the YS-150 and YS-170.  This configuration
;   is documented in the YS-150 and YS-170 GS sheets and 
;   manuals.  Having a programmed version of the configuration
;   is useful when operationing conditions require a modification
;   of this commonly applied configuration.
;
;   The program is a general cascade loop configuration capable 
;   of doing :
;       1. PID control of input X1 with optional characterization
;       2. Remote set point (X2) with optional ratio and bias 
;          when in cascade
;       3. Output tracking of a remote signal (X3)
;       4. Feed-forward, output compensation based on X4
;       5. Optional square root extraction of all 4 inputs
;       6. Adjustable filtering of all 4 inputs
;       7. Self-tuning de-activation from front panel or external 
;          input
;       8. Remotely place controller into specified mode, manual, 
;          auto, casc.
;       9. Remote command to pre-position output to PMV1 value
;      10. Optional functions selected by setting values of 
;          parameters, P1 through P30 and DI-01.
;      11. Use as a three element boiler feedwater controller
;           Steam flow is the feed-forward input, X4
;           Boiler feedwater flow is the loop 2 PV, X3
;           Drum level is the loop 1 PV, X1

;   Copyright 1994, Dixon Control Systems, Inc.
;                    Houston, Texas
;                    Phone: 713-644-8025, Fax # 713-644-8285

; Read, filter and characterize the input signals

   ; The X1 input, Process Variable 1
LD X1        ; Load Input X1, PV for loop 1
LD P16       ; "1" to take square root
GIFSUB @SQRT ; If P16 > 50 per cent, then take go to SQRT
LD P18       ; "1" to use FX1 to linearize input
GIFSUB @FX1  ; Use FX1 to linearize if P18 > 50 per cent
LD P11       ; Filter time constant, per cent = seconds, PLG1
LAG1         ; First order, low pass filter
ST P1        ; Store filtered & characterized value of X1 into P1
ST Y3        ; Store Output Y3

; The X2 input, if used, the cascade input for remote set point
LD P9        ; If P9 is "0", less than 50 per cent, no cascade
NOT          ; If P9 is "1", greater than 50% process input X2
GIF @NOCAS   ; Jump if cascade input not used
LD X2        ; Load Input X2
ST T1        ; For de-bug store intermediate values
LD P10       ; "1" to take square root
GIFSUB @SQRT ; If P10 > 50 per cent, take square root
ST T2        ; For de-bug store intermediate values
LD P12       ; Filter time constant, per cent = seconds, CLG1
LAG2         ; First order, low pass filter
ST T3        ; For De-bug store intermediate values
LD P19       ; If P19 > 50 per cent, then do ratio & bias
NOT          ; Invert
GIF @NORB    ; Jump if no scaling, ratio or bias for cascade set 
             ; point
LD P5        ; P5, ratio span
*            ; P5*T3
LD P8        ; The cascade set point off-set, bias
+            ; P5*T3 + P8
ST T5        ; For De-bug, store in T5
@NORB        ; Jump to here if no ratio-bias to cascade set point
ST P2        ; Store filtered value of X1 into P2
ST CSV1      ; Store as Cascade set point
@NOCAS       ; Jump here if no cascade set point

;       Process X3, Process Variable 2
LD X3        ; Load input X3
LD P6        ; "1" to take square root
GIFSUB @SQRT ; If P28 > 50 per cent, then go to SQRT
LD P15       ; "1" to use FX1 to linearize input
GIFSUB @FX2  ; Use FX1 to linearize if P18 > 50 per cent
LD P13       ; Filter time constant, per cent = seconds, TLG
LAG3         ; First order, low pass filter
ST P3        ; Store filtered value of X3 into P3 

;       Process X4, feed-forward input, if P21 is a "1"
LD P21       ; If P21 > 50 per cent, then feed-forward
NOT          ; Invert
GIF @NOFFD   ; Jump if P21 is a "0", less than 50 per cent
LD X4        ; Load Input X4
ST T7        ; For de-bug, store in T7
LD P22       ; "1" to take square root
GIFSUB @SQRT ; If P22 > 50 per cent, then take go to SQRT
ST T14       ; Store in T14 for de-bug
LD P14       ; Filter time constant, per cent = seconds, FLG
LAG4         ; First order, low pass filter
ST T8        ; For de-bug, store in T8
LD P23       ; If P23 is "1", then Gain and bias for feed-forward
NOT          ; Invert
GIF @NOGNB   ; Jump if P23 is "0", less than 50 per cent
LD P24       ; Feed-forward gain
*            ; P24*(T8 or T9)
ST T10       ; Store for de-bug
LD P25       ; Feed-forward off-set, bias
+            ; P24*(T8 or T9) + P25
ST T11       ; Store for de-bug
@NOGNB       ; Jump to here if no gain bais
ST P4        ; Store filtered value of X4 into P4
@NOFFD       ; Jump to here if no feed-forward

; Set feed-forward value in FF1 if track flag T26 is on.
; P26 will set T26 if the loop is in manual.  Turning the
; feed-forward on or off in automatic will cause a jump so the 
; option is inhibited.
LD CAF2      ; "1" if loop 2 is in cascade
LD P26       ; "1" to activate, set from P register panel
OR           ; "1" in automatic or P26 is "1"
ST T23       ; Store for de-bug
LD CAF2      ; "1" if loop 2 is in cascade
NOT          ; "1" if loop 2 is in manual or automatic
LD P26       ; "1" to activate feed-forward
AND          ; "1" in manual if P26 is "1", begin Feed-forward
ST T24       ; Store for de-bug
LD T26       ; Feed-forward flag, "1" feed-forward is active
OR           ; Part of the latch to latch in feed-forward
ST T25       ; Store for de-bug
LD P21       ; Feed-forward not implemented flag,"1" to implement
LD T23       ; "0" to stop feed-forward, manual and P26 "0"
AND          ; "1" if P21 is "1" and latch are "1" 
AND          ; "1" to get feed-forward going
ST T26       ; The feed-forward flag
NOT          ; If T26 is "0" don't change FF1
GIF @FFDEAD  ; Jump if feed-forward is dead
LD P4        ; Feed-forward value after scaling, filtering etc.
ST FF1       ; Store value in feed-forward register
@FFDEAD      ; Jump to here if feed-forward processing off

;  When DI-01 is closed, actions will be taken based on
;   P27=1,2,or3,self-tune controlled by pf (1) or DI-01(2or3) 
;   P29=100, then go to manual if DI-1 is "1", closed
;   P29=200, then go to manual if DI-1 is closed, auto if open
;   P29=300, when DI-1 is closed, then auto output is PMV1
;   P29=400, when DI-1 is closed, open the cascade loop 
LD P29          ; P29 used to change remote auto/manual
NOT             ; NOT
GIF @NODI1      ; Jump if P29 = "0" (less than 50 per cent)
LD K11          ; Constant = to 150%
LD P29          ; P29
CMP             ; "1" if P29 between 50% and 150%
GIF @DI1AM      ; Jump if 50% < P29 < 150%
LD K12          ; Constant = to 250%
LD P29          ; P29
CMP             ; "1" if P29 between 150% and 250%
GIF @PMV1       ; Jump if 150% < P29 < 250%
LD K13          ; Constant = to 350%
LD P29          ; P29
CMP             ; "1" if P29 between 250% and 350%
GIF @EXTSP      ; Jump if 250% < P29 < 350%
LD K1           ; Constant = to 100%
ST OCF          ; set cascade Open/Closed Flag to "1"
GO @NODI1       ; Jump to end of this section
@EXTSP          ; Jump here if 250% < P29 < 350%
LD K1           ; Constant = to 100%
LD DI1          ; Digital Input 1
AND             ; AND
ST PMVF1        ; set Preset MV Flag, "1" uses PMV1 as output
ST CAMF1        ; set Auto/Manual Flag, "1" is Auto
GO @NODI1       ; Jump to end of this section
@PMV1           ; Jump here if 150% < P29 < 250%
LD DI1          ; Digital Input 1
NOT             ; NOT
ST CAMF1        ; Auto/Manual Flag, "1" is Auto
GO @NODI1       ; Jump to end of this section
@DI1AM          ; Jump here if 50% < P29 < 150%
LD DI1          ; Digital Input 1
NOT             ; NOT
GIF @NODI1      ; Jump to end of section if DI1 = 0
LD K2           ; K2=0
ST CAMF2        ; Set Auto/Manual Flag to Manual
@NODI1          ; Jump here

;  PID control
LD P3        ; Filtered and manipulated Input X3
LD P1        ; Filtered and manipulated Input X1
CSC          ; CaSCade controller block
ST Y1        ; Output Y1


LD P27       ; Self-tune, "0", Std.;"1",pf;"2" or "3",DI-01
NOT          ; Invert
GIF @ENDSELT ; Jump if self-tune not inhibited remotely
LD P27       ; "1" uses pf key; "2" or "3" uses DI-01 to 
             ; inhibit STC
LD K11       ; K11 = 1.5
CMP          ; "0", use pf key, "1", use DI-01
GIF @DISETS  ; Jump if DI-01 sets self-tune
LD T27       ; Load self-tune status
LD KY1       ; Load status of pf key
CCD1         ; One shot
EOR          ; Exclusive OR
ST T27       ; Set self-tuning status
NOT          ; Invert
ST STCSW     ; "1" to inhibit self-tuning
GO @ENDSEFT  ; Jump to end of self-tuning
@DISETS      ; Use DI-01 to set self-tuning
LD DI1       ; DI-01 "ON" or "OFF" based on P27 to self-tune
LD P27       ; "2" for DI01 closed for self-tune;"3" open
LD K10       ; K10 = 2
-            ; P27 - K2, 0 or 1
EOR          ; Exclusive OR
ST STCSW     ; Set self-tune flag setting
@ENDSEFT     ; End of self-tuning
LD STCSW     ; "1"=stop self-tuning
NOT          ; "0" indicates no self-tuning allowed
ST LP1       ; Store self-tune activity in LP1
@ENDSELT     ; End of self-tuning flag setting

; Set Discrete Outputs
LD PHF1      ; "1" if process high alarm
LD VLF1      ; VeLocity alarm Flag
LD PLF1      ; Process Low Flag
OR           ; OR
OR           ; OR
LD P30       ; Sets output sense, 100per cent for normally closed
EOR          ; Exclusive OR
ST DO1       ; set DI-01, high alarm output
LD DLF1      ; Deviation aLarm Flag
ST DO2       ; Digital Output 2
LD OCF       ; Open/Closed cascade Flag
LD P30       ; sets output sense,100 percent for normally closed
EOR          ; Exclusive OR
ST DO3       ; Digital Output 3
LD CAMF1     ; "0" if in manual, "1" in automatic or cascade
ST DO5       ; DO-05 open
LD CAF1      ; "1" in cascade, "0" in auto or manual
ST DO4       ; DO4 closed in cascade, open in manual or aut

END          ; **** End of Main Program ****

SUB @SQRT    ; Sub-program to take Square root of S1
LD P17       ; Typically P17 is 1%, cut-off on square root
SQTE         ; Take square root
RTN          ; **** Return to Main Program ****

SUB @FX1     ; Use look-up table FX1 to process S1
FX1          ; Function Generator, FX1
RTN          ; **** Return to Main Program ****

SUB @FX2     ; Use look-up table FX2 to process S1
FX2          ; Function Generator FX2, set values from panel
RTN          ; **** Return to Main Program ****
